class PageMissingError(Exception):
    def __init__(self, title, xml):
        self.title = title
        self.xml = xml

    def __str__(self):
        return "page '%s' not found" % self.title


class ExportAbortedError(Exception):
    def __init__(self, index):
        self.index = index

    def __str__(self):
        return "Export from '%s' did not return anything." % self.index


class FileSizeError(Exception):
    def __init__(self, file, size):
        self.file = file
        self.size = size

    def __str__(self):
        return "File '%s' size is not match '%s'." % (self.file, self.size)


class FileSha1Error(Exception):
    def __init__(self, file, sha1):
        self.file = file
        self.sha1 = sha1

    def __str__(self):
        return "File '%s' sha1 is not match '%s'." % (self.file, self.sha1)



import datetime
import hashlib
from pathlib import Path
import re
import sys
from typing import Optional, Union

from wikiteam3.dumpgenerator.config import Config

ALL_DUMPED_MARK = "all_dumped.mark"
UPLOADED_MARK = 'uploaded_to_IA.mark'


def underscore(text: str) -> str:
    """ replace(" ", "_") """
    return text.replace(" ", "_")

def space(text: str) -> str:
    """ replace("_", " ") """
    return text.replace("_", " ")


def clean_HTML(raw: str = "") -> str:
    """Extract only the real wiki content and remove rubbish
    This function is ONLY used to retrieve page titles
    and file names when no API is available
    DO NOT use this function to extract page content"""
    # different "tags" used by different MediaWiki versions to mark where
    # starts and ends content
    if re.search("<!-- bodytext -->", raw):
        raw = raw.split("<!-- bodytext -->")[1].split("<!-- /bodytext -->")[0]
    elif re.search("<!-- start content -->", raw):
        raw = raw.split("<!-- start content -->")[1].split("<!-- end content -->")[0]
    elif re.search("<!-- Begin Content Area -->", raw):
        raw = raw.split("<!-- Begin Content Area -->")[1].split(
            "<!-- End Content Area -->"
        )[0]
    elif re.search("<!-- content -->", raw):
        raw = raw.split("<!-- content -->")[1].split("<!-- mw_content -->")[0]
    elif re.search(r'<article id="WikiaMainContent" class="WikiaMainContent">', raw):
        raw = raw.split('<article id="WikiaMainContent" class="WikiaMainContent">')[
            1
        ].split("</article>")[0]
    elif re.search("<body class=", raw):
        raw = raw.split("<body class=")[1].split('<div class="printfooter">')[0]
    else:
        print(raw[:250])
        print("This wiki doesn't use marks to split content")
        sys.exit(1)
    return raw


def undo_HTML_entities(text: str = "") -> str:
    """Undo some HTML codes"""

    # i guess only < > & " ' need conversion
    # http://www.w3schools.com/html/html_entities.asp
    text = re.sub("&lt;", "<", text)
    text = re.sub("&gt;", ">", text)
    text = re.sub("&amp;", "&", text)
    text = re.sub("&quot;", '"', text)
    text = re.sub("&#039;", "'", text)

    return text


def remove_IP(raw: str = "") -> str:
    """Remove IP from HTML comments <!-- -->"""

    raw = re.sub(r"\d+\.\d+\.\d+\.\d+", "0.0.0.0", raw)
    # http://www.juniper.net/techpubs/software/erx/erx50x/swconfig-routing-vol1/html/ipv6-config5.html
    # weird cases as :: are not included
    raw = re.sub(
        r"(?i)[\da-f]{0,4}:[\da-f]{0,4}:[\da-f]{0,4}:[\da-f]{0,4}:[\da-f]{0,4}:[\da-f]{0,4}:[\da-f]{0,4}:[\da-f]{0,4}",
        "0:0:0:0:0:0:0:0",
        raw,
    )

    return raw


def clean_XML(xml: str = "") -> str:
    """Trim redundant info from the XML however it comes"""
    # do not touch XML codification, leave AS IS
    # EDIT 2022: we are making this explicitly Unicode
    # for Windows compatibility.
    # If the encoding has to stay as is, we'll have
    # to change all the file encodings, as well.

    if re.search(r"</siteinfo>\n", xml):
        xml = xml.split("</siteinfo>\n")[1]
    if re.search(r"</mediawiki>", xml):
        xml = xml.split("</mediawiki>")[0]
    return xml

def sha1sum(path: Union[str, Path]) -> str:
    """Return the SHA1 hash of a file"""
    if isinstance(path, str):
        path = Path(path).expanduser().resolve()

    if not path.is_file():
        raise FileNotFoundError(f"File {path} does not exist or is not a file")

    sha1 = hashlib.sha1()
    with open(path, "rb") as f:
        while True:
            data = f.read(65536)
            if not data:
                break
            sha1.update(data)
    return sha1.hexdigest()

def mark_as_done(config: Config, mark: str, msg: Optional[str] = None):
    done_path = f"{config.path}/{mark}"
    if Path(done_path).exists():
        return
    with open(done_path, "w") as f:
        today = datetime.datetime.isoformat(datetime.datetime.now())
        f.write(f"{today}: {msg or ''}\n")

    return True

def is_markfile_exists(config: Config, mark: str) -> bool:
    return (Path(config.path)/ mark).exists()



import re
import sys
import time
from typing import Any, Dict, Generator

import requests

from wikiteam3.dumpgenerator.exceptions import ExportAbortedError, PageMissingError
from wikiteam3.dumpgenerator.api import handle_StatusCode
from wikiteam3.dumpgenerator.log import log_error
from wikiteam3.utils import uprint
from wikiteam3.dumpgenerator.config import Config


HISTORY_MIN_CHUNKSIZE = 2
""" To loop over all the revisions, we need to retrieve at least 2 revisions at a time. """


def getXMLPageCore(params: Dict, config: Config, session: requests.Session) -> str:
    """"""
    # returns a XML containing params['limit'] revisions (or current only), ending in </mediawiki>
    # if retrieving params['limit'] revisions fails, returns a current only version
    # if all fail, returns the empty string

    # if not headers:
    #     headers = session.headers
    xml = ""
    c = 0
    maxseconds = 100  # max seconds to wait in a single sleeping
    maxretries = config.retries  # x retries and skip
    increment = 20  # increment every retry

    while not re.search(r"</mediawiki>", xml):
        if c > 0 and c < maxretries:
            wait = (
                increment if increment * c < maxseconds else maxseconds
            )  # incremental until maxseconds
            print(
                f'    In attempt {c}, XML for "{params["pages"]}" is wrong. Waiting {wait} seconds and reloading...'
            )
            time.sleep(wait)
            # reducing server load requesting smallest chunks (if curonly then
            # limit = 1 from mother function)
            if params["limit"] > 1:
                # NOTE: if limit is float and betwennt 0 to 1, the MW backend will force-int it to 0
                new_limit: int = params["limit"] // 2  # half
                if new_limit < HISTORY_MIN_CHUNKSIZE:
                    new_limit = HISTORY_MIN_CHUNKSIZE
                
                # set new limit
                if new_limit != params["limit"]:
                    print(
                        f'    Reducing the chunksize of revisions to retrieve from {params["limit"]} to {new_limit}'
                    )
                    params["limit"] = new_limit
        if c >= maxretries:
            print("    We have retried %d times" % (c))
            print(
                '    MediaWiki error for "%s", network error or whatever...'
                % (params["pages"])
            )
            if config.failfast:
                print("Exit, it will be for another time")
                sys.exit(1)
            # If it's not already what we tried: our last chance, preserve only the last revision...
            # config.curonly means that the whole dump is configured to save only the last,
            # params['curonly'] should mean that we've already tried this
            # fallback, because it's set by the following if and passed to
            # getXMLPageCore
            if not config.curonly and "curonly" not in params:
                print("    Trying to save only the last revision for this page...")
                params["curonly"] = 1
                log_error(
                    config=config, to_stdout=True,
                    text='Error while retrieving the full history of "%s". Trying to save only the last revision for this page'
                    % (params["pages"]),
                )
                return getXMLPageCore(
                    params=params, config=config, session=session
                )
            else:
                print("    Saving in the errors log, and skipping...")
                log_error(
                    config=config, to_stdout=True,
                    text='Error while retrieving the last revision of "%s". Skipping.'
                    % (params["pages"]),
                )
                raise ExportAbortedError(config.index)
                return ""  # empty xml
        # FIXME HANDLE HTTP Errors HERE
        try:
            r = session.post(
                url=config.index, params=params, timeout=10
            )
            handle_StatusCode(r)
            xml = r.text
        except requests.exceptions.ConnectionError as e:
            print("    Connection error: %s" % (str(e.args[0])))
            xml = ""
        except requests.exceptions.ReadTimeout as e:
            print("    Read timeout: %s" % (str(e.args[0])))
            xml = ""
        c += 1

    return xml


def getXMLPageWithExport(config: Config, title: str,
                         *, verbose=True, session: requests.Session
                         ) -> Generator[str, None, None]:
    """Get the full history (or current only) of a page"""

    # if server errors occurs while retrieving the full page history,
    # it may return [oldest OK versions] + last version, excluding middle revisions,
    # so it would be partialy truncated
    # http://www.mediawiki.org/wiki/Manual_talk:Parameters_to_Special:Export#Parameters_no_longer_in_use.3F

    limit = 1000
    truncated = False
    title_ = title.replace(" ", "_")
    # do not convert & into %26, title_ = re.sub('&', '%26', title_)

    params: Dict[str, Any]
    if config.export:
        params = {"title": config.export, "pages": title_, "action": "submit"}
    else:
        params = {"title": "Special:Export", "pages": title_, "action": "submit"}
    if config.curonly:
        params["curonly"] = 1
        params["limit"] = 1
    else:
        params["offset"] = "1"  # 1 always < 2000s
        params["limit"] = limit
    # in other case, do not set params['templates']
    if config.templates:
        params["templates"] = 1

    xml = getXMLPageCore(params=params, config=config, session=session)
    if xml == "":
        raise ExportAbortedError(config.index)
    if "</page>" not in xml:
        raise PageMissingError(params["title"], xml)


    yield xml.split("</page>")[0]

    # if complete history, check if this page history has > limit edits, if so, retrieve all using offset if available
    # else, warning about Special:Export truncating large page histories
    r_timestamp = "<timestamp>([^<]+)</timestamp>"

    edit_count = 0
    edit_count += len(re.findall(r_timestamp, xml))

    # search for timestamps in xml to avoid analysing empty pages like
    # Special:Allpages and the random one
    if not config.curonly and re.search(r_timestamp, xml):
        while not truncated and params["offset"]:  # next chunk
            # get the last timestamp from the acum XML
            params["offset"] = re.findall(r_timestamp, xml)[-1]
            try:
                xml2 = getXMLPageCore(params=params, config=config, session=session)
            except MemoryError:
                print("The page's history exceeds our memory, halving limit.")
                params["limit"] = params["limit"] / 2
                continue

            # are there more edits in this next XML chunk or no <page></page>?
            if re.findall(r_timestamp, xml2):
                if re.findall(r_timestamp, xml2)[-1] == params["offset"]:
                    # again the same XML, this wiki does not support params in
                    # Special:Export, offer complete XML up to X edits (usually
                    # 1000)
                    print(
                        "ATTENTION: This wiki does not allow some parameters in Special:Export, therefore pages with large histories may be truncated"
                    )
                    truncated = True
                    break
                else:
                    """</namespaces>
                    </siteinfo>
                    <page>
                    <title>Main Page</title>
                    <id>15580374</id>
                    <restrictions>edit=sysop:move=sysop</restrictions> (?)
                    <revision>
                        <id>418009832</id>
                        <timestamp>2011-03-09T19:57:06Z</timestamp>
                        <contributor>
                    """
                    # offset is OK in this wiki, merge with the previous chunk
                    # of this page history and continue
                    try:
                        xml2 = xml2.split("</page>")[0]
                        yield "  <revision>" + (
                            "<revision>".join(xml2.split("<revision>")[1:])
                        )
                    except MemoryError:
                        "The page's history exceeds our memory, halving limit."
                        params["limit"] = params["limit"] / 2
                        continue
                    xml = xml2
                    edit_count += len(re.findall(r_timestamp, xml))
            else:
                params["offset"] = ""  # no more edits in this page history
    yield "</page>\n"

    if verbose:
        if edit_count == 1:
            uprint("    %s, 1 edit" % (title.strip()))
        else:
            uprint("    %s, %d edits" % (title.strip(), edit_count))



import os
import re
from typing import Dict, List
import pytest
import requests
from pathlib import Path

from wikiteam3.dumpgenerator.dump.image.html_regexs import REGEX_CANDIDATES

ONLINE = True

HTML_DIR = Path("test/data/html_regexs")
os.makedirs(HTML_DIR, exist_ok=True)

def prepare_raws_from_urls(urls: Dict[str, str]):
    sess = requests.Session()
    raws: Dict[str, str] = {}
    for site, url in urls.items():
        try:
            resp = sess.get(url, timeout=10, allow_redirects=True)
        except Exception as e:
            pytest.warns(UserWarning, match=f"Could not fetch {url}: {e}")
            continue

        if resp.status_code == 200:
            raws[url] = resp.text
            if not os.path.exists(HTML_DIR / f"{site}.html"):
                with open(HTML_DIR / f"{site}.html", "w", encoding="utf-8") as f:
                    f.write(resp.text)
        else:
            pytest.warns(UserWarning, match=f"Could not fetch {url}: status_code: {resp.status_code}")

    return raws

class TestRegexs:
    class TestRegexsOnline:
        listFiles_urls = {
            # site-date: url , `limit=` for counting the number of matches
            "group0.mediawiki.demo.save-web.org_mediawiki-1.16.5-20230701": "http://group0.mediawiki.demo.save-web.org/mediawiki-1.16.5/index.php?title=特殊:文件列表&limit=2",
            "group2.mediawiki.demo.save-web.org_mediawiki-1.39.1-20230701": "http://group2.mediawiki.demo.save-web.org/mediawiki-1.39.1/index.php?title=Special:ListFiles&limit=1",
            "archiveteam.org-20230701": "https://wiki.archiveteam.org/index.php?title=Special:ListFiles&sort=byname&limit=7",
            "wiki.othing.xyz-20230701": "https://wiki.othing.xyz/index.php?title=Special:ListFiles&sort=byname",
            "mediawiki.org-20230701": "https://www.mediawiki.org/w/index.php?title=Special:ListFiles&sort=byname&limit=7",
            "asoiaf.fandom.com-20230701": "https://asoiaf.fandom.com/zh/wiki/Special:文件列表?sort=byname&limit=7",
            
            # only for local testing:
            # "commons.moegirl.org.cn-20230701": "https://commons.moegirl.org.cn/index.php?title=Special:ListFiles&sort=byname&limit=7",
            # # login required:
            # "group0.mediawiki.demo.save-web.org_mediawiki-1.23.17-20230701": "http://group0.mediawiki.demo.save-web.org/mediawiki-1.23.17/index.php?title=Special:文件列表&limit=1",
            # "group1.mediawiki.demo.save-web.org_mediawiki-1.27.7-20230701": "http://group1.mediawiki.demo.save-web.org/mediawiki-1.27.7/index.php?title=Special:ListFiles&limit=2",
        }
        raws: Dict[str, str] = {}
        def test_online(self):
            if not ONLINE:
                pytest.skip("Online test skipped")
            self.raws = prepare_raws_from_urls(self.listFiles_urls)
            assert len(self.raws) != 0, "Could not fetch any of the URLs"
            for url, raw in self.raws.items():
                best_matched = 0
                regexp_best = None

                for index, regexp in enumerate(REGEX_CANDIDATES):
                    _count = len(re.findall(regexp, raw))
                    if _count > best_matched:
                        best_matched = _count
                        regexp_best = regexp

                assert regexp_best is not None, f"Could not find a proper regexp to parse the HTML for {url} (online)"

                if "limit=" in url:
                    limit = int(url.split("limit=")[-1])
                    assert len(re.findall(regexp_best, raw)) == limit, f"Could not find {limit} matches for {url} (online)"


    class TestRegexsOffline:
        html_files = os.listdir(HTML_DIR)
        raws: Dict[str, str] = {}
        for html_file in html_files:
            with open(HTML_DIR / html_file, "r", encoding="utf-8") as f:
                raws[html_file] = f.read()
        assert len(raws) != 0, f"Could not find any HTML files in {HTML_DIR}"

        def test_offline(self):
            assert len(self.raws) != 0, "Could not fetch any of the URLs"
            for site, raw in self.raws.items():
                best_matched = 0
                regexp_best = None

                for index, regexp in enumerate(REGEX_CANDIDATES):
                    _count = len(re.findall(regexp, raw))
                    if _count > best_matched:
                        best_matched = _count
                        regexp_best = regexp

                assert regexp_best is not None, f"Could not find a proper regexp to parse the HTML for {site} (local)"



import os

import requests

from wikiteam3.dumpgenerator.cli import Delay
from wikiteam3.utils import remove_IP
from wikiteam3.dumpgenerator.config import Config


def save_SpecialVersion(config: Config, session: requests.Session):
    """Save Special:Version as .html, to preserve extensions details"""

    if os.path.exists("%s/SpecialVersion.html" % (config.path)):
        print("SpecialVersion.html exists, do not overwrite")
    else:
        print("Downloading Special:Version with extensions and other related info")
        r = session.post(
            url=config.index, params={"title": "Special:Version"}, timeout=10
        )
        raw = r.text
        Delay(config=config)
        raw = remove_IP(raw=raw)
        with open(
            "%s/SpecialVersion.html" % (config.path), "w", encoding="utf-8"
        ) as outfile:
            outfile.write(raw)




import json
import os

import requests

from wikiteam3.dumpgenerator.cli import Delay
from wikiteam3.dumpgenerator.api import get_JSON
from wikiteam3.dumpgenerator.config import Config


def save_siteinfo(config: Config, session: requests.Session):
    """Save a file with site info"""

    if config.api:
        if os.path.exists("%s/siteinfo.json" % (config.path)):
            print("siteinfo.json exists, do not overwrite")
        else:
            print("Downloading site info as siteinfo.json")

            # MediaWiki 1.13+
            r = session.get(
                url=config.api,
                params={
                    "action": "query",
                    "meta": "siteinfo",
                    "siprop": "general|namespaces|statistics|dbrepllag|interwikimap|namespacealiases|specialpagealiases|usergroups|extensions|skins|magicwords|fileextensions|rightsinfo",
                    "sinumberingroup": 1,
                    "format": "json",
                },
                timeout=10,
            )
            # MediaWiki 1.11-1.12
            if not "query" in get_JSON(r):
                r = session.get(
                    url=config.api,
                    params={
                        "action": "query",
                        "meta": "siteinfo",
                        "siprop": "general|namespaces|statistics|dbrepllag|interwikimap",
                        "format": "json",
                    },
                    timeout=10,
                )
            # MediaWiki 1.8-1.10
            if not "query" in get_JSON(r):
                r = session.get(
                    url=config.api,
                    params={
                        "action": "query",
                        "meta": "siteinfo",
                        "siprop": "general|namespaces",
                        "format": "json",
                    },
                    timeout=10,
                )
            result = get_JSON(r)
            Delay(config=config)
            with open(
                "%s/siteinfo.json" % (config.path), "w", encoding="utf-8"
            ) as outfile:
                outfile.write(json.dumps(result, indent=4, sort_keys=True, ensure_ascii=False))



from io import TextIOWrapper
import re
import sys
from typing import NoReturn

import lxml.etree
import requests

from wikiteam3.dumpgenerator.cli import Delay
from wikiteam3.utils import url2prefix_from_config
from wikiteam3.dumpgenerator.exceptions import PageMissingError
from wikiteam3.dumpgenerator.log import log_error
from wikiteam3.dumpgenerator.api.page_titles import read_titles
from wikiteam3.dumpgenerator.dump.page.xmlexport.page_xml import get_XML_page
from wikiteam3.dumpgenerator.config import Config
from wikiteam3.utils import clean_XML, undo_HTML_entities
from wikiteam3.dumpgenerator.dump.xmldump.xml_header import getXMLHeader
from wikiteam3.dumpgenerator.dump.page.xmlrev.xml_revisions import getXMLRevisions
from wikiteam3.dumpgenerator.dump.xmldump.xml_truncate import truncateXMLDump, parse_last_page_chunk


def doXMLRevisionDump(config: Config, session: requests.Session, xmlfile: TextIOWrapper,
                      lastPage=None, useAllrevisions: bool=False):
    try:
        r_timestamp = r"<timestamp>([^<]+)</timestamp>"
        r_arvcontinue = r'<page arvcontinue="(.*?)">'

        lastArvcontinue = None
        for xml in getXMLRevisions(config=config, session=session, lastPage=lastPage, useAllrevision=useAllrevisions):
            numrevs = len(re.findall(r_timestamp, xml))
            arvcontinueRe = re.findall(r_arvcontinue, xml)
            if arvcontinueRe:
                curArvcontinue = arvcontinueRe[0]
                if lastArvcontinue != curArvcontinue:
                    Delay(config=config)
                    lastArvcontinue = curArvcontinue
            # Due to how generators work, it's expected this may be less
            xml = clean_XML(xml=xml)
            xmlfile.write(xml)

            xmltitle = re.search(r"<title>([^<]+)</title>", xml)
            title = undo_HTML_entities(text=xmltitle.group(1))
            print(f'{title}, {numrevs} edits (--xmlrevisions)')
            # Delay(config=config)
    except AttributeError as e:
        print(e)
        print("This API library version is not working")
        sys.exit(1)
    except UnicodeEncodeError as e:
        print(e)

def doXMLExportDump(config: Config, session: requests.Session, xmlfile: TextIOWrapper, lastPage=None):
    print(
        '\nRetrieving the XML for every page\n'
    )

    lock = True
    start = None
    if lastPage is not None:
        try:
            start = lastPage.find('title').text
        except Exception:
            print("Failed to find title in last trunk XML: %s" % (lxml.etree.tostring(lastPage)))
            raise
    else:
        # requested complete xml dump
        lock = False

    c = 1
    for title in read_titles(config, session=session, start=start):
        if not title:
            continue
        if title == start:  # start downloading from start, included
            lock = False
        if lock:
            continue
        Delay(config=config)
        if c % 10 == 0:
            print(f"\n->  Downloaded {c} pages\n")
        try:
            for xml in get_XML_page(config=config, title=title, session=session):
                xml = clean_XML(xml=xml)
                xmlfile.write(xml)
        except PageMissingError:
            log_error(
                config=config, to_stdout=True,
                text='The page "%s" was missing in the wiki (probably deleted)'
                     % title,
            )
        # here, XML is a correct <page> </page> chunk or
        # an empty string due to a deleted page (logged in errors log) or
        # an empty string due to an error while retrieving the page from server
        # (logged in errors log)
        c += 1


def generate_XML_dump(config: Config, resume=False, *, session: requests.Session):
    """Generates a XML dump for a list of titles or from revision IDs"""

    header, config = getXMLHeader(config=config, session=session)
    footer = "</mediawiki>\n"  # new line at the end
    xmlfilename = "{}-{}-{}.xml".format(
        url2prefix_from_config(config=config),
        config.date,
        "current" if config.curonly else "history",
    )
    xmlfile = None

    lastPage = None
    lastPageChunk = None
    # start != None, means we are resuming a XML dump
    if resume:
        print(
            "Removing the last chunk of past XML dump: it is probably incomplete."
        )
        # truncate XML dump if it already exists
        lastPageChunk = truncateXMLDump(f"{config.path}/{xmlfilename}")
        if not lastPageChunk.strip():
            print("Last page chunk is NULL, we'll directly start a new dump!")
            resume = False
            lastPage = None
        else:
            lastPage = parse_last_page_chunk(lastPageChunk)
            if lastPage is None:
                print("Failed to parse last page chunk: \n%s" % lastPageChunk)
                print("Cannot resume, exiting now!")
                sys.exit(1)

        print("WARNING: will try to start the download...")
        xmlfile = open(
            "{}/{}".format(config.path, xmlfilename), "a", encoding="utf-8"
        )
    else:
        print("\nRetrieving the XML for every page from the beginning\n")
        xmlfile = open(
            "{}/{}".format(config.path, xmlfilename), "w", encoding="utf-8"
        )
        xmlfile.write(header)

    if config.xmlrevisions and not config.xmlrevisions_page:
        doXMLRevisionDump(config, session, xmlfile, lastPage, useAllrevisions=True)
    elif config.xmlrevisions and config.xmlrevisions_page:
        doXMLRevisionDump(config, session, xmlfile, lastPage, useAllrevisions=False)
    else:  # --xml
        doXMLExportDump(config, session, xmlfile, lastPage)
    xmlfile.write(footer)
    xmlfile.close()
    print("XML dump saved at...", xmlfilename)
    return xmlfilename



from wikiteam3.dumpgenerator.dump import DumpGenerator


def main():
    DumpGenerator()


def main_deprecated(): # poetry scripts dumpgenerator = "wikiteam3.dumpgenerator:main_deprecated"
    import warnings
    warnings.warn("dumpgenerator is deprecated, use wikiteam3dumpgenerator instead", DeprecationWarning)
    main()



import datetime

from wikiteam3.dumpgenerator.version import getVersion


def welcome():
    message = ""
    """Opening message"""
    message += "#" * 73
    message += "\n"
    welcome_string = "# Welcome to DumpGenerator %s by WikiTeam3 (GPL v3)" % (
        getVersion()
    )
    welcome_string += " " * (73 - len(welcome_string) - 1) + "#"
    message += welcome_string
    message += "\n"
    message += (
        "# More info at: https://github.com/saveweb/wikiteam3                  #"
    )
    message += "\n"
    message += (
        "# Copyright (C) 2011-%d WikiTeam developers                           #\n"
        % (datetime.datetime.now().year)
    )
    message += "#" * 73

    return message


def bye():
    """Closing message"""
    print("")
    print("---> Done <---")
    print("")
    print("If this is a public wiki, please, consider publishing this dump.")
    print("Do it yourself as explained in:")
    print("  https://github.com/WikiTeam/wikiteam/wiki/Tutorial#Publishing_the_dump")
    print("")
    print("Good luck! Bye!")
    print("")




import os
from pathlib import Path
import subprocess
import time
from typing import Tuple, Union
import warnings


class ZstdCompressor:
    DEFAULT_LEVEL = 17
    MIN_VERSION = (1, 4, 8)
    def __init__(self, zstd_bin: str = "zstd"):
        """ versionCheck: check if zstd version is >= 1.4.8 """
        version = self.versionNumber(zstd_bin=zstd_bin)
        assert version >= self.MIN_VERSION, f"zstd version must be >= {self.MIN_VERSION}"
        # if v1.5.0-v1.5.4
        if (1, 5, 0) <= version <= (1, 5, 4):
            warnings.warn("your zstd version is between 1.5.0 and 1.5.4, which is not recommended due to a rare corruption bug in high compression mode, PLEASE UPGRADE TO 1.5.5+")
            print("sleeping for 20 seconds to let you read this message")
            time.sleep(20)

    @staticmethod
    def versionNumber(zstd_bin: str = "zstd") -> Tuple[int, int, int]:
        """
        Return runtime library version, the value is (`MAJOR`, `MINOR`, `RELEASE`).
        """
        rettext =  subprocess.check_output([zstd_bin, "-q", "-V"], shell=False).decode().strip()
        # 1.5.5
        ret_versions = [int(x) for x in rettext.split(".")]
        assert len(ret_versions) == 3
        return tuple(ret_versions)

    @staticmethod
    def compress_file(path: Union[str, Path], *, level: int = DEFAULT_LEVEL, zstd_bin: str = "zstd"):
        ''' Compress path into path.zst and return the absolute path to the compressed file.

        we set -T0 to use all cores, --long=31 to use 2^31 (2GB) window size

        level:
            - 1 -> fast
            - ...
            - 19 -> high
            - ... (ultra mode)
            - 22 -> best
        '''
        if isinstance(path, str):
            path = Path(path)
        path = path.resolve() # remove trailing slash

        compressed_path = path.parent / (path.name + ".zst") # path + ".zst"
        compressed_path = compressed_path.resolve()

        compressing_temp_path = path.parent / (path.name + ".zst.tmp") # path + ".zst.tmp"
        compressing_temp_path = compressing_temp_path.resolve()

        if compressed_path.exists():
            print(f"File {compressed_path} already exists. Skip compressing.")
            return compressed_path

        cmd =  [zstd_bin, "-T0","-v", "--compress", "--force", "--long=31"]
        if level >= 20:
            cmd.append("--ultra")
        cmd.extend([f"-{level}", str(path), "-o", str(compressing_temp_path)])

        subprocess.run(cmd)
        assert compressing_temp_path.exists()
        # move tmp file to final file
        os.rename(compressing_temp_path, compressed_path)
        return compressed_path

    @staticmethod
    def test_integrity(path: Union[str, Path], zstd_bin: str = "zstd") -> bool:
        ''' Test if path is a valid zstd compressed file. '''
        if isinstance(path, str):
            path = Path(path)
        path = path.resolve()
        r = subprocess.run([zstd_bin,"-vv", "-d", "-t", "--long=31", str(path)])
        return r.returncode == 0

class SevenZipCompressor:
    def __init__(self, bin7z: str = "7z"):
        retcode = subprocess.call([bin7z, "-h"], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        if retcode:
            raise FileNotFoundError(f"7z binary not found at {bin7z}")

    @staticmethod
    def compress_dir(dir_path: Union[str, Path], bin7z: str = "7z", level: int = 0):
        ''' Compress dir_path into dump_dir.7z and return the resolved path to the compressed file. 
        
        level:
            - 0 -> only pack, no compression
            - 1 -> fast
            - ...
            - 9 -> ultra
        '''
        if isinstance(dir_path, str):
            dir_path = Path(dir_path)
        dir_path = dir_path.resolve() # remove trailing slash

        archive_path = dir_path.parent / (dir_path.name + ".7z") # dir_path + ".7z"
        archive_path = archive_path.resolve()

        archive_temp_path = dir_path.parent  / (dir_path.name + ".7z.tmp") # dir_path + ".7z.tmp"
        archive_temp_path = archive_temp_path.resolve()

        if archive_path.exists():
            print(f"File {archive_path} already exists. Skip compressing.")
            return archive_path

        if level:
            cmds = [bin7z, "a", "-t7z", "-m0=lzma2", f"-mx={level}", "-scsUTF-8",
                "-md=64m", "-ms=off"]
        else: # level == 0
            assert level == 0
            cmds = [bin7z, "a", "-t7z", f"-mx={level}", "-scsUTF-8", "-ms=off"]
        cmds.extend([str(archive_temp_path), str(dir_path)])

        r = subprocess.run(cmds, check=True)

        assert archive_temp_path.exists()
        # move tmp file to final file
        os.rename(archive_temp_path, archive_path)
        assert archive_path == archive_path.resolve()
        return archive_path
    
    @staticmethod
    def test_integrity(path: Union[str, Path], bin7z: str = "7z") -> bool:
        ''' Test if path is a valid 7z archive. '''
        if isinstance(path, str):
            path = Path(path)
        path = path.resolve()
        r = subprocess.run([bin7z, "t", str(path)])
        return r.returncode == 0

if __name__ == "__main__":
    ZstdCompressor()
    SevenZipCompressor()